@@ -7,6 +7,7 @@ Utility for creating a Python repl.
     embed(globals(), locals(), vi_mode=False)
 
 """
+
 from __future__ import annotations
 
 import asyncio
@@ -18,7 +19,8 @@ import traceback
 import types
 import warnings
 from dis import COMPILER_FLAG_NAMES
-from typing import Any, Callable, ContextManager, Iterable
+from pathlib import Path
+from typing import Any, Callable, ContextManager, Iterable, NoReturn, Sequence
 
 from prompt_toolkit.formatted_text import OneStyleAndTextTuple
 from prompt_toolkit.patch_stdout import patch_stdout as patch_stdout_context
@@ -38,7 +40,15 @@ try:
 except ImportError:
     PyCF_ALLOW_TOP_LEVEL_AWAIT = 0
 
-__all__ = ["PythonRepl", "enable_deprecation_warnings", "run_config", "embed"]
+
+__all__ = [
+    "PythonRepl",
+    "enable_deprecation_warnings",
+    "run_config",
+    "embed",
+    "exit",
+    "ReplExit",
+]
 
 
 def _get_coroutine_flag() -> int | None:
@@ -63,7 +73,7 @@ def _has_coroutine_flag(code: types.CodeType) -> bool:
 
 class PythonRepl(PythonInput):
     def __init__(self, *a, **kw) -> None:
-        self._startup_paths = kw.pop("startup_paths", None)
+        self._startup_paths: Sequence[str | Path] | None = kw.pop("startup_paths", None)
         super().__init__(*a, **kw)
         self._load_start_paths()
 
@@ -89,9 +99,16 @@ class PythonRepl(PythonInput):
                 raise
             except SystemExit:
                 raise
+            except ReplExit:
+                raise
             except BaseException as e:
                 self._handle_exception(e)
             else:
+                if isinstance(result, exit):
+                    # When `exit` is evaluated without parentheses.
+                    # Automatically trigger the `ReplExit` exception.
+                    raise ReplExit
+
                 # Print.
                 if result is not None:
                     self._show_result(result)
@@ -153,7 +170,10 @@ class PythonRepl(PythonInput):
                     continue
 
                 # Run it; display the result (or errors if applicable).
-                self.run_and_show_expression(text)
+                try:
+                    self.run_and_show_expression(text)
+                except ReplExit:
+                    return
         finally:
             if self.terminal_title:
                 clear_title()
@@ -347,7 +367,7 @@ class PythonRepl(PythonInput):
     def get_compiler_flags(self) -> int:
         return super().get_compiler_flags() | PyCF_ALLOW_TOP_LEVEL_AWAIT
 
-    def _compile_with_flags(self, code: str, mode: str):
+    def _compile_with_flags(self, code: str, mode: str) -> Any:
         "Compile code with the right compiler flags."
         return compile(
             code,
@@ -381,6 +401,7 @@ class PythonRepl(PythonInput):
             return self
 
         globals["get_ptpython"] = get_ptpython
+        globals["exit"] = exit()
 
     def _remove_from_namespace(self) -> None:
         """
@@ -433,9 +454,10 @@ def run_config(repl: PythonInput, config_file: str | None = None) -> None:
         input("\nPress ENTER to continue...")
 
     # Check whether this file exists.
-    if not os.path.exists(config_file) and explicit_config_file:
-        print("Impossible to read %r" % config_file)
-        enter_to_continue()
+    if not os.path.exists(config_file):
+        if explicit_config_file:
+            print(f"Impossible to read {config_file}")
+            enter_to_continue()
         return
 
     # Run the config file in an empty namespace.
@@ -456,14 +478,37 @@ def run_config(repl: PythonInput, config_file: str | None = None) -> None:
         enter_to_continue()
 
 
+class exit:
+    """
+    Exit the ptpython REPL.
+    """
+
+    # This custom exit function ensures that the `embed` function returns from
+    # where we are embedded, and Python doesn't close `sys.stdin` like
+    # the default `exit` from `_sitebuiltins.Quitter` does.
+
+    def __call__(self) -> NoReturn:
+        raise ReplExit
+
+    def __repr__(self) -> str:
+        # (Same message as the built-in Python REPL.)
+        return "Use exit() or Ctrl-D (i.e. EOF) to exit"
+
+
+class ReplExit(Exception):
+    """
+    Exception raised by ptpython's exit function.
+    """
+
+
 def embed(
-    globals=None,
-    locals=None,
+    globals: dict[str, Any] | None = None,
+    locals: dict[str, Any] | None = None,
     configure: Callable[[PythonRepl], None] | None = None,
     vi_mode: bool = False,
     history_filename: str | None = None,
     title: str | None = None,
-    startup_paths=None,
+    startup_paths: Sequence[str | Path] | None = None,
     patch_stdout: bool = False,
     return_asyncio_coroutine: bool = False,
 ) -> None:
@@ -492,10 +537,10 @@ def embed(
 
     locals = locals or globals
 
-    def get_globals():
+    def get_globals() -> dict[str, Any]:
         return globals
 
-    def get_locals():
+    def get_locals() -> dict[str, Any]:
         return locals
 
     # Create REPL.
